@@ -1,21 +1,29 @@
-#' Pluck or chuck a single element from a vector or environment
+#' Safely get or set an element deep within a nested data structure
 #'
-#' `pluck()` and `chuck()` implement a generalised form of `[[` that
-#' allow you to index deeply and flexibly into data structures.
-#' `pluck()` consistently returns `NULL` when an element does not
-#' exist, `chuck()` always throws an error in that case.
+#' @description
+#' `pluck()` implements a generalised form of `[[` that allow you to index
+#' deeply and flexibly into data structures. It always succeeds, returning
+#' `.default` if the index you are trying to access does not exist or is `NULL`.
+#'
+#' `pluck<-()` is the assignment equivalent, allowing you to modify an object
+#' deep within a nested data structure.
+#'
+#' `pluck_exists()` tells you whether or not an object exists using the
+#' same rules as pluck (i.e. a `NULL` element is equivalent to an absent
+#' element).
 #'
 #' @param .x,x A vector or environment
 #' @param ... A list of accessors for indexing into the object. Can be
-#'   an integer position, a string name, or an accessor function
+#'   an positive integer, a negative integer (to index from the right),
+#'   a string (to index into names), or an accessor function
 #'   (except for the assignment variants which only support names and
 #'   positions). If the object being indexed is an S4 object,
 #'   accessing it by name will return the corresponding slot.
 #'
-#'   These dots support [tidy dots][rlang::list2] features. In
-#'   particular, if your accessors are stored in a list, you can
-#'   splice that in with `!!!`.
-#' @param .default Value to use if target is empty or absent.
+#'   [Dynamic dots][rlang::dyn-dots] are supported. In particular, if
+#'   your accessors are stored in a list, you can splice that in with
+#'   `!!!`.
+#' @param .default Value to use if target is `NULL` or absent.
 #'
 #' @details
 #' * You can pluck or chuck with standard accessors like integer
@@ -31,32 +39,34 @@
 #' * These accessors never partial-match. This is unlike `$` which
 #'   will select the `disp` object if you write `mtcars$di`.
 #'
-#'
 #' @seealso [attr_getter()] for creating attribute getters suitable
 #'   for use with `pluck()` and `chuck()`. [modify_in()] for
 #'   applying a function to a pluck location.
+#' @export
 #' @examples
 #' # Let's create a list of data structures:
 #' obj1 <- list("a", list(1, elt = "foo"))
 #' obj2 <- list("b", list(2, elt = "bar"))
 #' x <- list(obj1, obj2)
 #'
-#'
 #' # pluck() provides a way of retrieving objects from such data
 #' # structures using a combination of numeric positions, vector or
 #' # list names, and accessor functions.
 #'
 #' # Numeric positions index into the list by position, just like `[[`:
 #' pluck(x, 1)
-#' x[[1]]
+#' # same as x[[1]]
+#'
+#' # Index from the back
+#' pluck(x, -1)
+#' # same as x[[2]]
 #'
 #' pluck(x, 1, 2)
-#' x[[1]][[2]]
+#' # same as x[[1]][[2]]
 #'
 #' # Supply names to index into named vectors:
 #' pluck(x, 1, 2, "elt")
-#' x[[1]][[2]][["elt"]]
-#'
+#' # same as x[[1]][[2]][["elt"]]
 #'
 #' # By default, pluck() consistently returns `NULL` when an element
 #' # does not exist:
@@ -66,43 +76,13 @@
 #' # You can also supply a default value for non-existing elements:
 #' pluck(x, 10, .default = NA)
 #'
-#' # If you prefer to consistently fail for non-existing elements, use
-#' # the opinionated variant chuck():
-#' chuck(x, 1)
-#' try(chuck(x, 10))
-#' try(chuck(x, 1, 10))
-#'
-#'
 #' # The map() functions use pluck() by default to retrieve multiple
 #' # values from a list:
-#' map(x, 2)
-#'
-#' # Pass multiple indexes with a list:
-#' map(x, list(2, "elt"))
-#'
-#' # This is equivalent to:
-#' map(x, pluck, 2, "elt")
-#'
-#' # You can also supply a default:
-#' map(x, list(2, "elt", 10), .default = "superb default")
-#'
-#' # Or use the strict variant:
-#' try(map(x, chuck, 2, "elt", 10))
-#'
-#'
-#' # You can also assign a value in a pluck location with pluck<-:
-#' pluck(x, 2, 2, "elt") <- "quuux"
-#' x
-#'
-#' # This is a shortcut for the prefix function assign_in():
-#' y <- assign_in(x, list(2, 2, "elt"), value = "QUUUX")
-#' y
-#'
+#' map_chr(x, 1)
+#' map_int(x, c(2, 1))
 #'
 #' # pluck() also supports accessor functions:
 #' my_element <- function(x) x[[2]]$elt
-#'
-#' # The accessor can then be passed to pluck:
 #' pluck(x, 1, my_element)
 #' pluck(x, 2, my_element)
 #'
@@ -112,30 +92,12 @@
 #' # expression:
 #' my_element(x[[1]])
 #'
-#'
 #' # If you have a list of accessors, you can splice those in with `!!!`:
 #' idx <- list(1, my_element)
 #' pluck(x, !!!idx)
-#' @export
 pluck <- function(.x, ..., .default = NULL) {
-  .Call(
-    pluck_impl,
-    x = .x,
-    index = list2(...),
-    missing = .default,
-    strict = FALSE
-  )
-}
-#' @rdname pluck
-#' @export
-chuck <- function(.x, ...) {
-  .Call(
-    pluck_impl,
-    x = .x,
-    index = list2(...),
-    missing = NULL,
-    strict = TRUE
-  )
+  check_dots_unnamed()
+  pluck_raw(.x, list2(...), .default = .default)
 }
 
 #' @rdname pluck
@@ -145,30 +107,56 @@ chuck <- function(.x, ...) {
   assign_in(.x, list2(...), value)
 }
 
-reduce_subset_call <- function(init, idx) {
-  if (!length(idx)) {
-    abort("Can't pluck-assign without pluck locations")
-  }
-  reduce(idx, subset_call, .init = init)
-}
-subset_call <- function(x, idx) {
-  if (!is_index(idx)) {
-    type <- friendly_type_of(idx)
-    abort(sprintf("The pluck-assign indices must be names or positions, not %s", type))
-  }
-  call("[[", x, idx)
+#' @rdname pluck
+#' @export
+pluck_exists <- function(.x, ...) {
+  check_dots_unnamed()
+
+  !is_zap(pluck_raw(.x, list2(...), .default = zap()))
 }
 
-is_index <- function(x) {
-  if (is.object(x)) {
-    return(FALSE)
-  }
-  if (!typeof(x) %in% c("character", "integer", "double")) {
-    return(FALSE)
-  }
-  length(x) == 1
+pluck_raw <- function(.x, index, .default = NULL) {
+  .Call(
+    pluck_impl,
+    x = .x,
+    index = index,
+    missing = .default,
+    strict = FALSE
+  )
 }
 
+#' Get an element deep within a nested data structure, failing if it doesn't
+#' exist
+#'
+#' `chuck()` implements a generalised form of `[[` that allow you to index
+#' deeply and flexibly into data structures. If the index you are trying to
+#' access does not exist (or is `NULL`), it will throw (i.e. chuck) an error.
+#'
+#' @seealso [pluck()] for a quiet equivalent.
+#' @inheritParams pluck
+#' @export
+#' @examples
+#' x <- list(a = 1, b = 2)
+#'
+#' # When indexing an element that doesn't exist `[[` sometimes returns NULL:
+#' x[["y"]]
+#' # and sometimes errors:
+#' try(x[[3]])
+#'
+#' # chuck() consistently errors:
+#' try(chuck(x, "y"))
+#' try(chuck(x, 3))
+chuck <- function(.x, ...) {
+  check_dots_unnamed()
+
+  .Call(
+    pluck_impl,
+    x = .x,
+    index = list2(...),
+    missing = NULL,
+    strict = TRUE
+  )
+}
 
 #' Create an attribute getter function
 #'
