@@ -1,5 +1,3 @@
-context("imap")
-
 x <- 1:3 %>% set_names()
 
 test_that("imap is special case of map2", {
@@ -7,7 +5,7 @@ test_that("imap is special case of map2", {
 })
 
 test_that("imap always returns a list", {
-  expect_is(imap(x, paste), "list")
+  expect_bare(imap(x, paste), "list")
 })
 
 test_that("atomic vector imap works", {
@@ -15,12 +13,7 @@ test_that("atomic vector imap works", {
   expect_length(imap_chr(x, paste), 3)
   expect_equal(imap_int(x, ~ .x + as.integer(.y)), x * 2)
   expect_equal(imap_dbl(x, ~ .x + as.numeric(.y)), x * 2)
-  expect_equal(imap_raw(as.raw(12), rawShift), rawShift(as.raw(12), 1) )
-})
-
-test_that("data frame imap works", {
-  skip_if_not_installed("dplyr")
-  expect_identical(imap_dfc(x, paste), imap_dfr(x, paste))
+  expect_equal(imap_vec(x, ~ .x + as.numeric(.y)), x * 2)
 })
 
 test_that("iwalk returns invisibly", {
